@@ -18,7 +18,7 @@ import Hledger.Utils
 
 
 -- characters than can't be in a non-quoted commodity symbol
-nonsimplecommoditychars = "0123456789-.@;\n \"{}" :: String
+nonsimplecommoditychars = "0123456789-.@;\n \"{}=" :: String
 
 quoteCommoditySymbolIfNeeded s | any (`elem` nonsimplecommoditychars) s = "\"" ++ s ++ "\""
                                | otherwise = s
