@@ -22,7 +22,7 @@ import Control.Exception hiding (try)
 import Control.Monad
 import Control.Monad.Error
 -- import Test.HUnit
-import Data.Char (toLower, isDigit)
+import Data.Char (toLower, isDigit, isSpace)
 import Data.List
 import Data.Maybe
 import Data.Ord
@@ -85,7 +85,7 @@ readJournalFromCsv mrulesfile csvfile csvdata =
 
   -- parse csv
   records <- (either throwerr id . validateCsv) `fmap` parseCsv csvfile csvdata
-  dbg 1 $ ppShow $ take 3 records
+  return $ dbg "" $ take 3 records
 
   -- identify header lines
   -- let (headerlines, datalines) = identifyHeaderLines records
@@ -98,7 +98,7 @@ readJournalFromCsv mrulesfile csvfile csvdata =
    then hPrintf stderr "creating default conversion rules file %s, edit this file for better results\n" rulesfile
    else hPrintf stderr "using conversion rules file %s\n" rulesfile
   rules <- either (throwerr.show) id `fmap` parseRulesFile rulesfile
-  dbg 1 $ ppShow rules
+  return $ dbg "" rules
 
   -- apply skip directive
   let headerlines = maybe 0 oneorerror $ getDirective "skip" rules
@@ -319,7 +319,7 @@ getDirective directivename = lookup directivename . rdirectives
 
 parseRulesFile :: FilePath -> IO (Either ParseError CsvRules)
 parseRulesFile f = do
-  s <- readFile' f
+  s <- readFile' f >>= expandIncludes
   let rules = parseCsvRules f s
   return $ case rules of
              Left e -> Left e
@@ -329,6 +329,19 @@ parseRulesFile f = do
   where
     toParseError s = newErrorMessage (Message s) (initialPos "")
 
+-- | Pre-parse csv rules to interpolate included files, recursively.
+-- This is a cheap hack to avoid rewriting the existing parser.
+expandIncludes :: String -> IO String
+expandIncludes s = do
+  let (ls,rest) = break (isPrefixOf "include") $ lines s
+  case rest of
+    [] -> return $ unlines ls
+    (('i':'n':'c':'l':'u':'d':'e':f):ls') -> do
+      let f' = dropWhile isSpace f
+      included <- readFile f' >>= expandIncludes
+      return $ unlines [unlines ls, included, unlines ls']
+    ls' -> return $ unlines $ ls ++ ls'   -- should never get here
+
 parseCsvRules :: FilePath -> String -> Either ParseError CsvRules
 -- parseCsvRules rulesfile s = runParser csvrulesfile nullrules{baseAccount=takeBaseName rulesfile} rulesfile s
 parseCsvRules rulesfile s =
